#=======================================================#
#                                                       #
#   Project:    SNA for CJ                              #
#   Author:     Andrew V. Papachristos                  #
#   Copyright:  2015                                    #
#														                            #
#	  Module:     Data										                #
#														                            #
#   Tasks:      Setting up RStudio Session				      #
#				        Importing Matrix Network Data           #
#               Converting a Matrix into a Network      #
#               Importing Edgelist Network Data         #
#               Converting an Edgelist into a Network   #  
#               Attaching Attributes to a Network       #
#               Importing Two-mode Event Data           # 
#               Converting Event Data into Two Networks #
#               Review                                  #
#                                                       #  
#=======================================================#

#--------------------------------------------------------------------

#   1.  Setting up RStudio Session 

# Remember that every line following the pound sign (#) in this script file is a note. Commands have their own lines, no pound signs at the beginning, and they will appear in a different color within your script. To execute a command from a script in RStudio, select that command or commands and click on the run button at the top of the script. Alternatively, the keyboard shortcut for running a command is command + return in Macs and control + enter in Windows.

# Check the contents of the working directory using the dir() command. Printed in the Console pane below will be a list of all the items available in the working directory. You should see the name of this R script as well as the 4 csv files described in the Module 2: Data Lab.

dir()

# Open the igraph library so that the igraph tools and commands are available for this RStudio session. Participants should have installed the igraph package in the Module 1: Introduction RStudio. If the following command produces an error message, then return to the Module 1: Introduction RStudio and follow the instructions for installing the igraph package. If the following command successfully opens igraph, the igraph library will remain open for the duration of the session until RStudio is closed. 

library(igraph)





#--------------------------------------------------------------------

#   2.  Importing Network Data from a Matrix

# Now that this RStudio session is set up, we will first import relational data using the sociomatrix format. The Siren Network is a real example of a criminal network from Montreal and is currently in sociomatrix format.

# Importing csv files into RStudio requires the read.csv command. Before running the full command below, let's first review all the parts of this command. Arguments for a command are located in the parentheses after the command, are separated by commas, and permit users to control various options of a command.

# The full command we will use is: m <- read.csv("matrix.csv", header=TRUE). The read.csv() is a command that imports a file in table or data frame format and creates a table or data frame within RStudio. There are several important arguments to the read.csv() command. The first argument in the read.csv() command is the name of the file to be loaded into RStudio. Because the working directory has been specified, the file name is only "matrix.csv" requiring the quotation marks. RStudio will look in the working directory folder and find the file that exactly matches this name. If the file we wanted to load was not in our working directory, we would need to specify the entire path name for RStudio to find the file on your computer. The next argument is header=TRUE. The default for the read.csv() command is header=FALSE, meaning that by default RStudio interprets the first row of a csv as data rather than as a header. We know from the previous section of this lab that the first row of "matrix.csv" is a header of numbers identifying nodes 1 through 44. Setting header=TRUE overrides the default and tells RStudio to treat row 1 as a header row rather than data. Note that the read.csv() command occurs to the right of the assignment arrow (<-). Remember from Module 1: Introduction RStudio that the assignment arrow assigns everything on the right of the arrow to the item located to the left of the arrowhead. On the left side of the arrowhead, we create a new object called "m" (m for matrix) that will hold the "matrix.csv" in RStudio. The new object "m" will equal the matrix from the csv file, but the difference is that "m" is internal to RStudio and the "matrix.csv" file is external to RStudio. It is possible to load a csv file with just the code to the right of the assignment arrow, but this will only load the data into RStudio and users will not be able to manipulate the loaded data easily.

m <- read.csv("matrix.csv", header=TRUE)

# The read.csv() command is part of the base utilities in RStudio, meaning that no additional library was needed to run this command. The code above is a simple version of a "read.csv command using most of the default settings. More advanced adjustments and arguments can be found using:

help(read.csv)

# To inspect how the data loaded into RStudio, we only need to run the name of the new object ("m"). The results in the Console pane will look a little strange because there is not enough width in the Console pane to print all 44 columns, but a matrix looking object should appear.

m

# Alternatively, locate "m" under Data in the Environment tab in the upper-right pane. Remember that the Environment tab keeps track of all the objects opened and created in RStudio. If this is a new RStudio session, "m" will be the only thing listed under Data. This pane includes some information on "m": 44 observations (rows) have 44 variables (columns). Clicking on m in the upper-right pane will open a snapshot of what the m object looks like in the upper-left pane. This action will cover up this script. Navigate back to this script by using the tabs located at the top of this upper-left pane. 

# As you learned in the Code School section of this RStudio lab, RStudio distinguishes between data frames and matrices. For igraph purposes, RStudio needs to interpret "m" as a matrix rather than a data frame. Check if RStudio thinks that the object "m" is a matrix by asking RStudio the true/false is.matrix() question using the following command:

is.matrix(m)

# If the console showed FALSE following the is.matrix() question command, then the object "m" is not ready for igraph. The object "m" needs to be converted to a matrix using the as.matrix() command. Note that the as.matrix() command below uses the assignment arrow. Remember from Module 1: Introduction RStudio that when we use the same object name on the left of the arrow we actually overwrite the original object. In other words, the command below replaces the old version of "m" created above with the read.csv() command  with a new matrix version of "m". Because we don't care about the non-matrix version of "m", it is fine to overwrite the old version with the new version.

m <- as.matrix(m)

# Notice how the object under Data in the upper-right pane has changed. Before the description of "m" read "44 obs. of 44 variables". Now the description of "m" reads "int [1:44, 1:44] 0 1 1 1 1 1 1 1 1 1...", meaning that there are 1 through 44 rows in "m" and 1 through 44 columns in "m", and the first row beings with 0 1 1 1 ... etc. 

# Now we can ask RStudio again if the object "m" is a matrix?

is.matrix(m)

# If TRUE then the matrix is ready to be converted into a network.





#--------------------------------------------------------------------

#   3.  Converting a Matrix into a Network

# Converting a matrix into a network requires a command from the igraph library. This command will not work if the library(igraph) command did not run correctly above when setting up this RStudio session. The command from igraph that converts a matrix into a network is called graph.adjacency(). The command below has 5 parts described from left to right: 

#       siren.network <- graph.adjacency(m, mode=c("undirected"))
# (1) Create a new object called "siren.network" to distinguish the network from the matrix "m". 
# (2) Assignment arrow will convert everything to the right of the arrow to the object located to the left of the arrow.
# (3) graph.adjacency() is the command in igraph needed to convert a matrix into a network. 
# (4) The first argument in the parentheses following the command is the name of the matrix to be converted into a network. Here the matrix is "m" because that is the name of the matrix we imported and converted above. 
# (5) The last argument in the parentheses separated by a comma is mode=c("undirected"), which  specifies how igraph should interpret the matrix. Networks can be "directed" (important distinction between sender and receiver) or "undirected" (no distinction between sender and receiver). We learned in the Module 2: Data Lab that this matrix is symmetric and undirected, so we want to specify the mode to be "undirected".
# Notice that there are two sets of parentheses following the graph.adjacency() command. The command will not work unless each parenthesis that is open is also closed. 
# Go ahead and run the command below to convert matrix "m" into the siren.network.

siren.network <- graph.adjacency(m, mode=c("undirected"))

# Notice that we have a new item located in the Environment pane. Click on the blue and white arrow button to the right of "siren.network" in the Environment pane, and it will open a list of all of the dyads in this network underneath.

# Inspect the new network object called "siren.network" in the Console pane by running just the name of the object.

siren.network

# The Console pane shows that "siren.network" is undirected (UN), has 44 nodes, and 103 edges. Yay! You have successfully converted a sociomatrix into a network.

# Below is the code for the most simple version of plotting a network image with igraph default settings. The code will generate a network image in the lower-left pane. Click the zoom button to see the network in a larger frame. It isn't a very nice looking network image, but we will save advanced plotting commands for the next Lab in Module 3: Visualization.

plot(siren.network)





#--------------------------------------------------------------------

#   4.  Loading Network Data from an Edgelist

# The Lab 2 Files folder contains a second example of network data that uses the edgelist format rather than the sociomatrix format. Edgelists are a different version of organizing information on relationships that can be converted into networks. Additionally, this edgelist example has an accompanying attribute file that contains information on the individuals.

# First, load the edgelist using the same read.csv() command as before. Name the RStudio object for this csv "e" for edgelist.

e <- read.csv("edgelist.csv", header=TRUE)

# Inspect the new object called "e" to check how the csv imported. 

e

# The object "e" contains a header plus 7 rows and 3 columns. The numbers 1-7 to the left of each row are row identifiers and not a separate column. Ignore the weight column for now as this will be addressed in Module 3: Visualization.

# We will use a different command in igraph that converts edglists to networks, so the object "e" should not be a matrix. We can ask RStudio if "e" is a matrix to make sure that we use the correct command in igraph.

is.matrix(e)

# If the answer if FALSE then we are ready to turn this edgelist into a network.





#--------------------------------------------------------------------

#   5.  Converting an Edgelist to a Network

# Returning to the commands available in the igraph library, the graph.data.frame() command can be used to convert an edgelist into a network (see also the graph.edgelist() command). Note that the elements of the command below are almost identical to the elements used with the graph.adjacency() command used above when we converted the "m" object into a network. However, the argument for directed or undirected networks is a bit more simple in the graph.data.frame() command The command below generates an undirected network from the edgelist object called "e" and stores the new network in a new object called "edge.network":

edge.network <- graph.data.frame(e, directed=FALSE)

# To inspect the conversion from edgelist to network simply run the name of the new network object, or click on the blue and white arrow to the left of the "edge.network" item in the Environment pane:

edge.network

# The object "edge.network" is undirected (UN), weighted (W), and has 6 nodes and 7 edges. The "edge.network" has a vertex-level piece of information called "name" identified by "(v/c)", and an edge-level piece of information called "weight" identified by "(e/n)". For now, we will ignore the edge-level information, as this will be introduced in the next module. 

# Plot the network using the default plot() settings in igraph. 

plot(edge.network)





#--------------------------------------------------------------------

#   6.  Attaching Attribute Data to a Network

# Often social network analysts want to include different information on the nodes of a network, which are called vertices in RStudio. This requires an additional file that includes attributes or individual-level data for each node in a network. The edgelist example above includes an accompanying file of attributes for each individual in the network. To begin, the csv file must be read into RStudio using the same command that imported the edgelist. We will call this attribute list "a" for attributes.

a<-read.csv("attributes.csv", header=TRUE)

a

# One option in the graph.data.frame() command is the argument vertices=. Vertices refer to the individuals or nodes in the network. The default argument is vertices=NULL, which is why the "edge.network" generated above did not require any attribute information for the nodes except for the names. The vertices= argument merges relational data and attribute data in one easy step. The command below has minor variations on the command above used to generate the "edge.network". To override the default of "NULL" for the vertices, we set the vertices argument to equal our attribute file, object "a". Because the names in the attribute file "a" match the names in the edgelist "e", igraph can match the individual-level information of gender, gpa, and age to the correct nodes. We will call the new network with the attribute information "g".

g <- graph.data.frame(e, directed=FALSE, vertices=a)

# Now inspecting the new network object "g" shows additional information under "attr:", which is short for attributes. The earlier version of this network only included "name" and "weight" in the attribute list. Now this network includes "name", "gender", "gpa", and "age" for each node or vertex (v/n) and "weight" for the edges (e/n). 

g

# Each of these attributes can be inspected using variations on the following command: 
#   V(g)$name 
# The V() command is internal to the igraph library and specifies vertex sequences or information on the nodes, "g" is the network we made with the attributes, the dollar sign ($) is an extraction device built into RStudio that can extract a column from a larger set of data, and the last piece is the name of the column or variable to be extracted. The commands below basically tell RStudio which column you want to see from the "g" network, but because all networks are made up of two levels of data (nodes and edges) you must also explain to RStudio that the variable of interest is located in V (nodes, individuals, vertices) not E (edges, ties, relationships). In the vertex sequences below stored in network "g", we will look at the four columns: name, gender, gpa, and age. 

V(g)$name
V(g)$gender
V(g)$gpa
V(g)$age

# Well that is all fine and good, but what can we do with these attributes now that they are assigned to the network. Now that we know that 2 nodes are female, we might want to change the color of those nodes to red. Advanced plotting commands will be introduced in the next Module to show how to include the attributes of the nodes as features in the network plot. At this stage, however, we can use some of the commands that we learned in Module 1: Introduction RStudio and apply them to the attributes in the network. Notice that the commands below add only one command and a set of parentheses to the beginning of the extraction commands presented above this paragraph.

table(V(g)$gender)
max(V(g)$gpa)
min(V(g)$gpa)
mean(V(g)$age)





#--------------------------------------------------------------------

#   7.  Importing Two-mode Event Data 

# The Lab 2 Files folder contains a csv file called "event.csv" based on some hypothetical arrests. This csv contains two-mode data in which individuals are connected through co-arrest events and co-arrest events are connected through shared individuals. Importing this csv file is simple and follows basically the same read.csv() command that we have used three times already in this lab. Generating the two-mode network and its one-mode projections are more complicated than the one-mode networks we made above using the sociomatrix and the edgelist.

# First we run the read.csv() command that we are familiar with by this point in the lab. There is one new addition to this command: colClasses="character". This addition tells RStudio that it should treat all of the columns as characters or strings rather than real numbers. This will become an important detail when converting the event list to a network. We will call the new object that we create from this event data "t" for two-mode.   

t <- read.csv("event.csv", header=TRUE, colClasses="character")

# Inspect what we just imported by running just the name of the new object or by clicking on the object "t" under Data in the Environment pane.

t

# Notice that there are two columns in object "t": person and event. There are 10 rows in object "t", and each row represents a tie between a person and an event. 

# We can also check that the columns successfully converted to characters rather than numbers using true/false questions. The extraction tool (i.e., dollar sign) is used to check each column separately. Let's check that the contents of these two columns are characters and not numbers.

is.character(t$person)
is.numeric(t$person)
is.character(t$event)
is.numeric(t$event)





#-----------------------------------------------------------------

#   8.  Converting Event Data into Two Networks

# Converting event data into networks is a bit more complicated than the one-mode networks we made above. The main difference is that a two-mode network requires a true/false list for the complete set of nodes that distinguishes between the two types of nodes--in our case the people (which will be our trues) and the events (which will be our falses). In this list each node needs to be listed only once. When you look at the columns in object "t", you can see that there are duplicates in each column because individuals are connected to multiple events and events have multiple individuals. We will use the identifiers in these two columns, but we need to make a list without the duplicates (e.g., 1, 3, e1, e5). With a small network like this, we could do this pretty quickly by hand, but suppose we were looking at an entire year of arrest data and had to make this list by hand. We might have thousands of entries. It is useful to automate some of these steps because we can reuse the commands in future projects on larger datasets with zero or only minor adjustments. First, we will use the unique() command to remove duplicate elements in our two-mode event data. We don't want to overwrite the dataset, so we will make 2 new lists: one for person and one for event.

person <- unique(t$person)
event <- unique(t$event)

# Check to see what the unique() commands above produced. 

person
event

# Now we have a list of 8 unique people and a list of 5 unique events. Notice our numerical ids for person are in quotation marks, which is correct because it means they are still characters and not numbers. 

# Next we need to stack these two lists on top of each other for one long list of 13 nodes (8 people + 5 events = 13 total nodes even though they are of two different types). Then we will assign true or false to each entry. We will do this by creating a data frame and we will call the new data frame object "tf.list" for true/false list. The first argument in the data.frame() command is name=c(event, person). This tells RStudio that the first column will be called "name" and will combine c() our two lists of unique events and unique persons that we generated above. For the second column named "type", we tell RStudio to replicate "false" for the length of the "event" and replicate "true" for the length of "person". The length argument tells RStudio to look at how many events and persons there are in our list, or how long that part of the list is. Using replicate rep() and length() together means that the true and false values will correctly align with our persons and events. Because we stacked event on top of person in the first name column we need to stack the falses for events on top of the trues for persons. In order to avoid any crazy conversions and introduce possible errors we will override the default setting that converts strings (a variable type) to factors (another variable type). (This is a rather advanced data management topic that is beyond the scope of this tutorial.) Our result below is the most complex command used in this tutorial thus far. Notice the six different sets of parentheses required to execute this command.

tf.list <- data.frame(name=c(event, person), type=c(rep(FALSE, length(event)), rep(TRUE, length(person))), stringsAsFactors=FALSE)

# But, the result from the complex command above is quite simple. Inspect it by running just the name of our new true/false list or by clicking on the object in the Environment pane.

tf.list

# This simple little list allows us to execute a two-mode network. We will call our new two-mode network "two.network." We use the graph.data.frame() command like we did above with the "g" network. Here our input is "t", our two-mode event data we imported from the csv. We will make this network undirected. We assign the vertices argument our new true false list, "tf.list". We used the vertices argument in network "g" when we wanted to attach attributes to the nodes, but for the two-mode data we essentially created our own attribute list within RStudio. The true/false list is similar to an attribute list, but it only contains one attribute. 

two.network <- graph.data.frame(t, directed=FALSE, vertices=tf.list)

# Inspect the results.

two.network
plot(two.network)

# We see that the "two.network" object is undirected and bipartite (two-mode), it contains 13 nodes, and 10 edges. It contains the name attribute and the type attribute. The plot of "two.network" is rather unimpressive as it does not distinguish between the two node types even after all of our hard work. It is difficult to see that this is even a two-mode network. We will focus on customizing the visualization of two-mode networks in Module 3.

# For now we want to create the two one-mode projections that come from this two-mode network. First, we run the bipartitie.projection() command. Remember that two-mode networks are also called bipartite networks. Let's call this new object "p" for projection.

p <- bipartite.projection(two.network)

# When you inspect "p", you will see that it contains two parts that can be extracted using the $ extraction tool: $proj1 containing 5 nodes and 2 ties and $proj2 containing 8 nodes and 6 ties.

p

# We can pull these two separate networks out of "p" and give them their own network object and name. The first projection is events and the second projection is persons. 

event.network <- p[[1]]
person.network <- p[[2]]

# Now we can easily inspect and plot these two networks that come out of two-mode event data.

event.network
person.network

plot(event.network)
plot(person.network)

# If we did this correctly, then one network should contain only events and the other network should contain only people. Remember that the people's connections to each other were based on their shared events, and the events' connections to each other were based on their shared people.





#-----------------------------------------------------------------

#   9.  Review

# Below are the commands from this script without the annotation. Users can review Module 2: Data RStudio commands quickly here. These commands also serve as a model to develop future work.

setwd("")         # Set working directory
library(igraph)   # Load igraph library

# Siren Network from a Matrix
m <- read.csv("matrix.csv", header=TRUE)
is.matrix(m)
m <- as.matrix(m)
is.matrix(m)
siren.network <- graph.adjacency(m, mode=c("undirected"))
siren.network
plot(siren.network)

# Example Edgelist and Attributes
e <- read.csv("edgelist.csv", header=TRUE)
e
edge.network <- graph.data.frame(e, directed=FALSE)
edge.network
plot(edge.network)
a<-read.csv("attributes.csv", header=TRUE)
a
g <- graph.data.frame(e, directed=FALSE, vertices=a)
g
V(g)$name
V(g)$gender
V(g)$gpa
V(g)$age

# Example Two-Mode Event Data
t <- read.csv("event.csv", header=TRUE, colClasses="character")
t
is.character(t$person)
is.numeric(t$person)
is.character(t$event)
is.numeric(t$event)
person <- unique(t$person)
event <- unique(t$event)
person
event
tf.list <- data.frame(name=c(event, person), type=c(rep(FALSE, length(event)), rep(TRUE, length(person))), stringsAsFactors=FALSE)
tf.list
two.network <- graph.data.frame(t, directed=FALSE, vertices=tf.list)
two.network
plot(two.network)
p <- bipartite.projection(two.network)
p
event.network <- p[[1]]
person.network <- p[[2]]
event.network
person.network
plot(event.network)
plot(person.network)




#-END----------------------------------------------------------------